@@ -1,5 +1,5 @@
 /* AbiSource Program Utilities
- * Copyright (C) 1998 AbiSource, Inc.
+ * Copyright (C) 1998-2000 AbiSource, Inc.
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,6 +24,7 @@
 #include "ut_assert.h"
 #include "ut_types.h"
 #include "ut_debugmsg.h"
+#include "ut_Win32OS.h"
 #include "ev_Keyboard.h"
 #include "ev_NamedVirtualKey.h"
 #include "ev_Win32Keyboard.h"
@@ -157,9 +158,6 @@
 // Note: instances of this class, but then so is the physical
 // Note: keyboard.
 
-static UT_Bool s_bMapped = UT_FALSE;
-static HKL s_hKeyboardLayout = 0;
-
 static EV_EditBits s_mapVirtualKeyCodeToNVK(WPARAM nVirtKey);
 
 /*****************************************************************/
@@ -168,14 +166,36 @@ static EV_EditBits s_mapVirtualKeyCodeToNVK(WPARAM nVirtKey);
 ev_Win32Keyboard::ev_Win32Keyboard(EV_EditEventMapper * pEEM)
 	: EV_Keyboard(pEEM)
 {
-	if (!s_bMapped)
-		remapKeyboard(GetKeyboardLayout(0));
-	s_bMapped = UT_TRUE;
+	m_hKeyboardLayout = 0;
+	m_iconv = (UT_iconv_t)-1;
+	remapKeyboard(GetKeyboardLayout(0));
+}
+
+ev_Win32Keyboard::~ev_Win32Keyboard()
+{
+	if( m_iconv != (UT_iconv_t)-1 )
+		UT_iconv_close( m_iconv );
 }
 
 void ev_Win32Keyboard::remapKeyboard(HKL hKeyboardLayout)
 {
-	s_hKeyboardLayout = hKeyboardLayout;
+	char  szCodePage[10];
+
+	if( m_iconv != (UT_iconv_t)-1 )
+	{
+		UT_iconv_close( m_iconv );
+		m_iconv = (UT_iconv_t)-1;
+	}
+	if( hKeyboardLayout != 0 )
+	{
+		strcpy( szCodePage, "CP" );
+		if( GetLocaleInfo( LOWORD( hKeyboardLayout ), LOCALE_IDEFAULTANSICODEPAGE, &szCodePage[2], sizeof( szCodePage ) / sizeof( szCodePage[0] ) - 2 ) )
+		{
+			m_iconv = UT_iconv_open( "UCS-2-INTERNAL", szCodePage );
+		}
+
+		m_hKeyboardLayout = hKeyboardLayout;
+	}
 }
 
 /*****************************************************************/
@@ -352,10 +372,11 @@ UT_Bool ev_Win32Keyboard::onKeyDown(AV_View * pView,
 
 	BYTE keyState[256];
 	BYTE buffer[2];
+	int count;
 
 	unsigned int scancode = (keyData & 0x00ff0000)>>16;
 	GetKeyboardState(keyState);
-	int count = ToAsciiEx(nVirtKey,scancode,keyState,(WORD *)buffer,0,s_hKeyboardLayout);
+	count = ToAsciiEx(nVirtKey,scancode,keyState,(WORD *)buffer,0,m_hKeyboardLayout);
 
 	if (count == -1)
 	{
@@ -384,7 +405,7 @@ UT_Bool ev_Win32Keyboard::onKeyDown(AV_View * pView,
 		keyState[VK_RCONTROL] &= ~0x80;
 		keyState[VK_RMENU] &= ~0x80;
 
-		count = ToAsciiEx(nVirtKey,scancode,keyState,(WORD *)buffer,0,s_hKeyboardLayout);
+		count = ToAsciiEx(nVirtKey,scancode,keyState,(WORD *)buffer,0,m_hKeyboardLayout);
 
 		if (count == 1)
 		{
@@ -418,7 +439,7 @@ UT_Bool ev_Win32Keyboard::onKeyDown(AV_View * pView,
 		_emitChar(pView,hWnd,iMsg,nVirtKey,keyData,buffer[0],0);
 		buffer[0] = buffer[1];
 	}
-	
+
 	// try to handle the remaining (or only) character.
 
 	if (buffer[0] >= 0x20)
@@ -456,7 +477,7 @@ UT_Bool ev_Win32Keyboard::onKeyDown(AV_View * pView,
 		keyState[VK_RCONTROL] &= ~0x80;
 		keyState[VK_RMENU] &= ~0x80;
 		
-		count = ToAsciiEx(nVirtKey,scancode,keyState,(WORD *)buffer,0,s_hKeyboardLayout);
+		count = ToAsciiEx(nVirtKey,scancode,keyState,(WORD *)buffer,0,m_hKeyboardLayout);
 
 		if (count == 1)
 		{
@@ -489,10 +510,23 @@ void ev_Win32Keyboard::_emitChar(AV_View * pView,
 	//			 (ems&EV_EMS_CONTROL)?"control":"",
 	//			 (ems&EV_EMS_ALT)?"alt":""));
 
-	UT_uint16 charData = (UT_uint16)b;
+	UT_uint16 charData;
+	if( m_iconv != (UT_iconv_t)-1 )
+	{
+		// convert to 8bit string and null terminate
+		size_t len_in, len_out;
+		const char *In = (const char *)&b;
+		char *Out = (char *)&charData;
+
+		len_in = 1;
+		len_out = 2;
+		UT_iconv( m_iconv, &In, &len_in, &Out, &len_out );
+	}
+	else
+		charData = b;
 
 	EV_EditMethod * pEM;
-	EV_EditEventMapperResult result = m_pEEM->Keystroke(EV_EKP_PRESS|ems|charData,&pEM);
+	EV_EditEventMapperResult result = m_pEEM->Keystroke(EV_EKP_PRESS|ems|b,&pEM);
 
 	switch (result)
 	{
