@@ -1,5 +1,5 @@
 /* AbiWord
- * Copyright (C) 1998 AbiSource, Inc.
+ * Copyright (C) 1998-2000 AbiSource, Inc.
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -63,18 +63,18 @@ UT_uint32 AP_BeOSFrame::getZoomPercentage(void)
         return ((AP_FrameData*)m_pData)->m_pG->getZoomPercentage();
 }            
 
-UT_Bool AP_BeOSFrame::_showDocument(UT_uint32 iZoom)
+UT_Error AP_BeOSFrame::_showDocument(UT_uint32 iZoom)
 {
 	if (!m_pDoc)
 	{
 		UT_DEBUGMSG(("Can't show a non-existent document\n"));
-		return UT_FALSE;
+		return UT_IE_FILENOTFOUND;
 	}
 
 	if (!((AP_FrameData*)m_pData))
 	{
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-		return UT_FALSE;
+		return UT_IE_IMPORTERROR;
 	}
 
 	GR_BeOSGraphics * pG = NULL;
@@ -157,7 +157,7 @@ UT_Bool AP_BeOSFrame::_showDocument(UT_uint32 iZoom)
 		// TODO a loop like this to help change the focus when the current
 		// TODO view changes.
 		
-		EV_BeOSToolbar * pBeOSToolbar = (EV_BeOSToolbar *)m_vecBeOSToolbars.getNthItem(k);
+		EV_BeOSToolbar * pBeOSToolbar = (EV_BeOSToolbar *)m_vecToolbars.getNthItem(k);
 		pBeOSToolbar->bindListenerToView(pView);
 	}
 
@@ -198,6 +198,8 @@ UT_Bool AP_BeOSFrame::_showDocument(UT_uint32 iZoom)
 	((AP_FrameData*)m_pData)->m_pTopRuler->setView(pView);
 	((AP_FrameData*)m_pData)->m_pLeftRuler->setView(pView);
 	//((AP_FrameData*)m_pData)->m_pStatusBar->setView(pView);
+
+	pView->setInsertMode(((AP_FrameData*)m_pData)->m_bInsertMode);
 	
 	m_pBeDocView->Window()->Lock();
 	m_pView->setWindowSize(m_pBeDocView->Bounds().Width(),
@@ -231,7 +233,7 @@ UT_Bool AP_BeOSFrame::_showDocument(UT_uint32 iZoom)
         ((AP_FrameData*)m_pData)->m_pLeftRuler->draw(NULL);
 	//((AP_FrameData*)m_pData)->m_pStatusBar->draw();
 
-	return UT_TRUE;
+	return UT_OK;
 
 Cleanup:
 	// clean up anything we created here
@@ -246,7 +248,7 @@ Cleanup:
 	UNREFP(m_pDoc);
 	m_pDoc = ((AP_FrameData*)m_pData)->m_pDocLayout->getDocument();
 
-	return UT_FALSE;
+	return UT_IE_ADDLISTENERERROR;
 }
 
 /*
@@ -368,8 +370,11 @@ UT_Bool AP_BeOSFrame::initialize()
 	//At this point in time the BeOS widgets are all
 	//realized so we should be able to go and attach
 	//the various input filters to them.
-	m_pBeOSKeyboard->synthesize(m_pBeOSApp, this);
-        m_pBeOSMouse->synthesize(m_pBeOSApp, this);
+	ev_BeOSKeyboard * pBeOSKeyboard = static_cast<ev_BeOSKeyboard *>(m_pKeyboard);
+	pBeOSKeyboard->synthesize(m_pBeOSApp, this);
+
+	ev_BeOSMouse * pBeOSMouse = static_cast<ev_BeOSMouse *>(m_pMouse);
+	pBeOSMouse->synthesize(m_pBeOSApp, this);
 
 	//Actually show the window to the world
 	m_pBeWin->Show();
@@ -384,7 +389,7 @@ UT_Bool AP_BeOSFrame::initFrameData(void)
 {
 	UT_ASSERT(!m_pData);
 
-	m_pData = new AP_FrameData();
+	m_pData = new AP_FrameData(m_pBeOSApp);
 	
 	return (m_pData ? UT_TRUE : UT_FALSE);
 }
@@ -395,7 +400,7 @@ void AP_BeOSFrame::killFrameData(void)
 	m_pData = NULL;
 }
 
-UT_Bool AP_BeOSFrame::_loadDocument(const char * szFilename, IEFileType ieft)
+UT_Error AP_BeOSFrame::_loadDocument(const char * szFilename, IEFileType ieft)
 {
 	// are we replacing another document?
 	if (m_pDoc)
@@ -417,30 +422,33 @@ UT_Bool AP_BeOSFrame::_loadDocument(const char * szFilename, IEFileType ieft)
 		goto ReplaceDocument;
 	}
 
-	if (pNewDoc->readFromFile(szFilename, ieft))
+	UT_Error err; 
+	err = pNewDoc->readFromFile(szFilename, ieft);
+	if (!err)
 		goto ReplaceDocument;
 	
 	UT_DEBUGMSG(("ap_Frame: could not open the file [%s]\n",szFilename));
 	UNREFP(pNewDoc);
-	return UT_FALSE;
+	return err;
 
 ReplaceDocument:
 	getApp()->forgetClones(this);
 
 	// NOTE: prior document is discarded in _showDocument()
 	m_pDoc = pNewDoc;
-	return UT_TRUE;
+	return UT_OK;
 }
 	
 XAP_Frame * AP_BeOSFrame::cloneFrame(void)
 {
 	AP_BeOSFrame * pClone = new AP_BeOSFrame(this);
+	UT_Error error = UT_OK;
 	ENSUREP(pClone);
 
 	if (!pClone->initialize())
 		goto Cleanup;
-
-	if (!pClone->_showDocument())
+	error = pClone->_showDocument();
+	if (error)
 		goto Cleanup;
 
 	pClone->show();
@@ -458,7 +466,7 @@ Cleanup:
 	return NULL;
 }
 
-UT_Bool AP_BeOSFrame::loadDocument(const char * szFilename, int ieft)
+UT_Error AP_BeOSFrame::loadDocument(const char * szFilename, int ieft)
 {
 	UT_Bool bUpdateClones;
 	UT_Vector vClones;
@@ -470,13 +478,15 @@ UT_Bool AP_BeOSFrame::loadDocument(const char * szFilename, int ieft)
 		pApp->getClones(&vClones, this);
 	}
 
-	if (! _loadDocument(szFilename,(IEFileType)ieft))
+	UT_Error err;
+	err = _loadDocument(szFilename, (IEFileType) ieft); 
+	if (err)
 	{
 		// we could not load the document.
 		// we cannot complain to the user here, we don't know
 		// if the app is fully up yet.  we force our caller
 		// to deal with the problem.
-		return UT_FALSE;
+		return err;
 	}
 
 	pApp->rememberFrame(this);
@@ -616,7 +626,7 @@ be_DocView *be_Window::_createDocumentWindow() {
         return(m_pbe_DocView);                                    
 }
 
-UT_Bool AP_BeOSFrame::_replaceDocument(AD_Document * pDoc)
+UT_Error AP_BeOSFrame::_replaceDocument(AD_Document * pDoc)
 {
 	// NOTE: prior document is discarded in _showDocument()
 	m_pDoc = REFP(pDoc);
