@@ -1,5 +1,5 @@
 /* AbiWord
- * Copyright (C) 1998 AbiSource, Inc.
+ * Copyright (C) 1998-2000 AbiSource, Inc.
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,14 +28,16 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
 #include "ut_debugmsg.h"
+#include "ut_string.h"
 #include "ut_misc.h"
-#include "ut_getopt.h"
 
 #include "xap_Args.h"
+#include "ap_Convert.h"
 #include "ap_UnixFrame.h"
 #include "ap_UnixApp.h"
 #include "sp_spell.h"
@@ -638,15 +640,16 @@ ReturnThisBuffer:
 static GtkWidget * wSplash = NULL;
 static GR_Image * pSplashImage = NULL;
 static GR_UnixGraphics * pUnixGraphics = NULL;
-static guint timeout_handler = 0;
 static UT_Bool firstExpose = FALSE;
 static UT_uint32 splashTimeoutValue = 0;
 
+static guint death_timeout_handler = 0;
+
 static gint s_hideSplash(gpointer /*data*/)
 {
 	if (wSplash)
 	{
-		gtk_timeout_remove(timeout_handler);
+		gtk_timeout_remove(death_timeout_handler);
 		gtk_widget_destroy(wSplash);
 		wSplash = NULL;
 		DELETEP(pUnixGraphics);
@@ -655,12 +658,6 @@ static gint s_hideSplash(gpointer /*data*/)
 	return TRUE;
 }
 
-// GTK never seems to let me have this event on a pop-up style window
-//static void s_key_event(GtkWidget * /*window*/, GdkEventKey * /*key*/)
-//{
-//	s_hideSplash(NULL);
-//}
-
 static void s_button_event(GtkWidget * /*window*/)
 {
 	s_hideSplash(NULL);
@@ -677,7 +674,8 @@ static gint s_drawingarea_expose(GtkWidget * /* widget */,
 		if (!firstExpose)
 		{
 			firstExpose = UT_TRUE;
-			timeout_handler = gtk_timeout_add(splashTimeoutValue, s_hideSplash, NULL);
+			// kill the window after splashTimeoutValue ms
+			death_timeout_handler = gtk_timeout_add(splashTimeoutValue, s_hideSplash, NULL);
 		}
 	}
 
@@ -733,8 +731,6 @@ GR_Image * AP_UnixApp::_showSplash(UT_uint32 delay)
 		gtk_widget_set_usize(da, iSplashWidth, iSplashHeight);
 		gtk_signal_connect(GTK_OBJECT(da), "expose_event",
 						   GTK_SIGNAL_FUNC(s_drawingarea_expose), NULL);
-//		gtk_signal_connect(GTK_OBJECT(da), "key_press_event",
-//						   GTK_SIGNAL_FUNC(s_key_event), NULL);
 		gtk_signal_connect(GTK_OBJECT(da), "button_press_event",
 						   GTK_SIGNAL_FUNC(s_button_event), NULL);
 		gtk_container_add(GTK_CONTAINER(frame), da);
@@ -745,13 +741,16 @@ GR_Image * AP_UnixApp::_showSplash(UT_uint32 delay)
 
 		// create the window so we can attach a GC to it
 		gtk_widget_show(wSplash);
-
+		
 		// create image context
 		pUnixGraphics = new GR_UnixGraphics(da->window, NULL);
 		pSplashImage = pUnixGraphics->createNewImage("splash", pBB, iSplashWidth, iSplashHeight);
 
 		// another for luck (to bring it up forward and paint)
 		gtk_widget_show(wSplash);
+
+		// trigger an expose event to get us started
+		s_drawingarea_expose(da, NULL);
 	}
 
 	DELETEP(pBB);
@@ -776,8 +775,29 @@ int AP_UnixApp::main(const char * szAppName, int argc, char ** argv)
 
 	XAP_Args Args = XAP_Args(argc,argv);
 
+	// Do a quick and dirty find for "-to"
+ 	UT_Bool bShowSplash = UT_TRUE;
+	UT_Bool bShowApp = UT_TRUE;
+ 	for (int k = 1; k < Args.m_argc; k++)
+ 		if (*Args.m_argv[k] == '-')
+ 			if (UT_stricmp(Args.m_argv[k],"-to") == 0)
+ 			{
+				bShowApp = UT_FALSE;
+ 				bShowSplash = UT_FALSE;
+ 				break;
+ 			}
+
+	// Do a quick and dirty find for "-show"
+ 	for (int k = 1; k < Args.m_argc; k++)
+ 		if (*Args.m_argv[k] == '-')
+ 			if (UT_stricmp(Args.m_argv[k],"-show") == 0)
+ 			{
+				bShowApp = UT_TRUE;
+ 				bShowSplash = UT_TRUE;
+ 				break;
+ 			}
+
 	// Do a quick and dirty find for "-nosplash"
-	UT_Bool bShowSplash = UT_TRUE;
 	for (int k = 1; k < Args.m_argc; k++)
 		if (*Args.m_argv[k] == '-')
 			if (UT_stricmp(Args.m_argv[k],"-nosplash") == 0)
@@ -794,7 +814,7 @@ int AP_UnixApp::main(const char * szAppName, int argc, char ** argv)
 	
 	if (bShowSplash)
 		_showSplash(2000);
-			
+
 	AP_UnixApp * pMyUnixApp = new AP_UnixApp(&Args, szAppName);
 
 	// if the initialize fails, we don't have icons, fonts, etc.
@@ -807,7 +827,7 @@ int AP_UnixApp::main(const char * szAppName, int argc, char ** argv)
 	// this function takes care of all the command line args.
 	// if some args are botched, it returns false and we should
 	// continue out the door.
-	if (pMyUnixApp->parseCommandLine())
+	if (pMyUnixApp->parseCommandLine() && bShowApp)
 	{
 		// turn over control to gtk
 		gtk_main();
@@ -835,7 +855,10 @@ UT_Bool AP_UnixApp::parseCommandLine(void)
 	int nFirstArg = 1;
 	int k;
 	int kWindowsOpened = 0;
-        
+	char *to = NULL;
+	int verbose = 1;
+	UT_Bool show = UT_FALSE;
+
 	for (k=nFirstArg; (k<m_pArgs->m_argc); k++)
 	{
 		if (*m_pArgs->m_argv[k] == '-')
@@ -858,7 +881,7 @@ UT_Bool AP_UnixApp::parseCommandLine(void)
 				// dump the string table in english as a template for translators.
 				// see abi/docs/AbiSource_Localization.abw for details.
 				AP_BuiltinStringSet * pBuiltinStringSet = new AP_BuiltinStringSet(this,AP_PREF_DEFAULT_StringSet);
-				pBuiltinStringSet->dumpBuiltinSet("EnUS.strings");
+				pBuiltinStringSet->dumpBuiltinSet("en-US.strings");
 				delete pBuiltinStringSet;
 #endif
 			}
@@ -898,6 +921,20 @@ UT_Bool AP_UnixApp::parseCommandLine(void)
 				// set the xap-level geometry for future frame use
 				setGeometry(x, y, width, height, f);
 			}
+			else if (UT_stricmp (m_pArgs->m_argv[k],"-to") == 0)
+			{
+				k++;
+				to = m_pArgs->m_argv[k];
+			}
+			else if (UT_stricmp (m_pArgs->m_argv[k], "-show") == 0)
+			{
+				show = UT_TRUE;
+			}
+			else if (UT_stricmp (m_pArgs->m_argv[k], "-verbose") == 0)
+			{
+				k++;
+				verbose = atoi (m_pArgs->m_argv[k]);
+			}
 			else
 			{
 				UT_DEBUGMSG(("Unknown command line option [%s]\n",m_pArgs->m_argv[k]));
@@ -910,30 +947,44 @@ UT_Bool AP_UnixApp::parseCommandLine(void)
 		else
 		{
 			// [filename]
-                        
-			AP_UnixFrame * pFirstUnixFrame = new AP_UnixFrame(this);
-			pFirstUnixFrame->initialize();
-			if (pFirstUnixFrame->loadDocument(m_pArgs->m_argv[k], IEFT_Unknown))
+ 			if (to) 
 			{
-				kWindowsOpened++;
-			}
-			else
-			{
-				// TODO: warn user that we couldn't open that file
+ 				AP_Convert * conv = new AP_Convert();
+ 				conv->setVerbose(verbose);
+ 				conv->convertTo(m_pArgs->m_argv[k], to);
+ 				delete conv;
+  			}
+  			else
+  			{
+				AP_UnixFrame * pFirstUnixFrame = new AP_UnixFrame(this);
+				pFirstUnixFrame->initialize();
+				UT_Error error = pFirstUnixFrame->loadDocument(m_pArgs->m_argv[k], IEFT_Unknown);
+				if (!error)
+				{
+					kWindowsOpened++;
+				}
+				else
+				{
+					// TODO: warn user that we couldn't open that file
 
 #if 1
-				// TODO we crash if we just delete this without putting something
-				// TODO in it, so let's go ahead and open an untitled document
-				// TODO for now.  this would cause us to get 2 untitled documents
-				// TODO if the user gave us 2 bogus pathnames....
-				kWindowsOpened++;
-				pFirstUnixFrame->loadDocument(NULL, IEFT_Unknown);
+					// TODO we crash if we just delete this without putting something
+					// TODO in it, so let's go ahead and open an untitled document
+					// TODO for now.  this would cause us to get 2 untitled documents
+					// TODO if the user gave us 2 bogus pathnames....
+					kWindowsOpened++;
+					pFirstUnixFrame->loadDocument(NULL, IEFT_Unknown);
 #else
-				delete pFirstUnixFrame;
+					delete pFirstUnixFrame;
 #endif
+				}
 			}
 		}
 	}
+						
+	// command-line conversion may not open any windows at all
+	if (to && !show)
+		return UT_TRUE;
 
 	if (kWindowsOpened == 0)
 	{
@@ -949,14 +1000,16 @@ UT_Bool AP_UnixApp::parseCommandLine(void)
 
 // TODO : MOVE THIS TO XP CODE!  This is a cut & paste job since each
 // TODO : platform _can_ have different options, and we didn't sort
-// TODO : out how to honor them correclty yet.  There is a copy of
+// TODO : out how to honor them correctly yet.  There is a copy of
 // TODO : this function in other platforms.
 
 void AP_UnixApp::_printUsage(void)
 {
 	// just print to stdout, not stderr
 	printf("\nUsage: %s [option]... [file]...\n\n", m_pArgs->m_argv[0]);
-
+	printf("  -to               The target format of the file (abw, zabw, rtf, txt, utf8, html, latex)");
+	printf("  -verbose          The verbosity level (0, 1, 2)");
+	printf("  -show             If you really want to start the GUI (even if you use the --to option)");
 #ifdef DEBUG
 	printf("  -dumpstrings      dump strings strings to file\n");
 #endif
