@@ -1,5 +1,5 @@
 /* AbiWord
- * Copyright (C) 1998 AbiSource, Inc.
+ * Copyright (C) 1998-2000 AbiSource, Inc.
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,7 +32,9 @@
 
 #include "ut_debugmsg.h"
 #include "ut_bytebuf.h"
+#include "ut_string.h"
 #include "xap_Args.h"
+#include "ap_Convert.h"
 #include "ap_Win32Frame.h"
 #include "ap_Win32App.h"
 #include "sp_spell.h"
@@ -320,7 +322,7 @@ void AP_Win32App::copyToClipboard(PD_DocumentRange * pDocRange)
 		if (pExpRtf)
 		{
 			UT_ByteBuf buf;
-			IEStatus status = pExpRtf->copyToBuffer(pDocRange,&buf);
+			UT_Error status = pExpRtf->copyToBuffer(pDocRange,&buf);
 			UT_Byte b = 0;
 			buf.append(&b,1);			// NULL terminate the string
 			m_pClipboard->addData(AP_CLIPBOARD_RTF,(UT_Byte *)buf.getPointer(0),buf.getLength());
@@ -335,7 +337,7 @@ void AP_Win32App::copyToClipboard(PD_DocumentRange * pDocRange)
 		if (pExpText)
 		{
 			UT_ByteBuf buf;
-			IEStatus status = pExpText->copyToBuffer(pDocRange,&buf);
+			UT_Error status = pExpText->copyToBuffer(pDocRange,&buf);
 			UT_Byte b = 0;
 			buf.append(&b,1);			// NULL terminate the string
 			m_pClipboard->addData(AP_CLIPBOARD_TEXTPLAIN_8BIT,(UT_Byte *)buf.getPointer(0),buf.getLength());
@@ -531,6 +533,30 @@ static GR_Image * _showSplash(HINSTANCE hInstance, XAP_Args * pArgs, const char
 	int k;
 	
 	// scan args for splash-related stuff
+    for (k=nFirstArg; (k<pArgs->m_argc); k++)
+	{
+		if (*pArgs->m_argv[k] == '-')
+		{
+			if (UT_stricmp(pArgs->m_argv[k],"-to") == 0)
+			{
+				bShowSplash = UT_FALSE;
+				break;
+			}
+		}
+	}	
+	
+    for (k=nFirstArg; (k<pArgs->m_argc); k++)
+	{
+		if (*pArgs->m_argv[k] == '-')
+		{
+			if (UT_stricmp(pArgs->m_argv[k],"-show") == 0)
+			{
+				bShowSplash = UT_TRUE;
+				break;
+			}
+		}
+	}	
+
 	for (k=nFirstArg; (k<pArgs->m_argc); k++)
 	{
 		if (*pArgs->m_argv[k] == '-')
@@ -602,7 +628,7 @@ static GR_Image * _showSplash(HINSTANCE hInstance, XAP_Args * pArgs, const char
 		UT_PNG_getDimensions(pBB, iSplashWidth, iSplashHeight);
 
 		// create a centered window the size of our bitmap
-		hwndSplash = CreateWindow(s_SplashWndClassName, 
+		hwndSplash = CreateWindowEx(WS_EX_TOOLWINDOW | WS_EX_TOPMOST,s_SplashWndClassName, 
 								  NULL, WS_POPUP | WS_BORDER,
 								  (rect.right  / 2) - (iSplashWidth  / 2),
 								  (rect.bottom / 2) - (iSplashHeight / 2),
@@ -619,7 +645,6 @@ static GR_Image * _showSplash(HINSTANCE hInstance, XAP_Args * pArgs, const char
 			DELETEP(pG);
 
 			// now bring the window up front & center
-			SetWindowPos(hwndSplash, HWND_TOPMOST, 0,0,0,0, SWP_NOMOVE | SWP_NOSIZE); 
 			ShowWindow(hwndSplash, SW_SHOWNORMAL);
 			UpdateWindow(hwndSplash);
 		}
@@ -634,9 +659,13 @@ Done:
 
 /*****************************************************************/
 
+typedef BOOL __declspec(dllimport) (CALLBACK *InitCommonControlsEx_fn)(LPINITCOMMONCONTROLSEX lpInitCtrls);
+
 int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance, 
 						 HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
 {
+	UT_Bool bShowApp = UT_TRUE;
+
 	// this is a static function and doesn't have a 'this' pointer.
 	MSG msg;
 
@@ -645,13 +674,31 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 	_CrtSetReportMode( _CRT_ASSERT, _CRTDBG_MODE_DEBUG );
 
 	// Ensure that common control DLL is loaded
-
-	INITCOMMONCONTROLSEX icex;
-	icex.dwSize = sizeof(INITCOMMONCONTROLSEX);
-	icex.dwICC = ICC_COOL_CLASSES | ICC_BAR_CLASSES		// load the rebar and toolbar
-				| ICC_TAB_CLASSES | ICC_UPDOWN_CLASS	// and tab and spin controls
-				;
-	InitCommonControlsEx(&icex);
+	HINSTANCE hinstCC = LoadLibrary("comctl32.dll");
+	UT_ASSERT(hinstCC);
+	InitCommonControlsEx_fn  pInitCommonControlsEx = NULL;
+	if( hinstCC != NULL )
+		pInitCommonControlsEx = (InitCommonControlsEx_fn)GetProcAddress( hinstCC, "InitCommonControlsEx" );
+	if( pInitCommonControlsEx != NULL )
+	{
+		INITCOMMONCONTROLSEX icex;
+		icex.dwSize = sizeof(INITCOMMONCONTROLSEX);
+		icex.dwICC = ICC_COOL_CLASSES | ICC_BAR_CLASSES 	// load the rebar and toolbar
+					| ICC_TAB_CLASSES | ICC_UPDOWN_CLASS	// and tab and spin controls
+					;
+		pInitCommonControlsEx(&icex);
+	}
+	else
+	{
+		InitCommonControls();
+		MessageBox(NULL,
+			"AbiWord is designed for a newer version of the system file COMCTL32.DLL\n"
+			"then the one currently on your system.\n"
+			"A solution to this problem is explained in the FAQ on AbiSource web site\n"
+			"\n\thttp://www.abisource.com\n\n"
+			"We hope this problem can be solved, until then you can use the program,\n"
+			"but the toolbar will be missing.", NULL, MB_OK);
+	}
 
 	// HACK: load least-common-denominator Rich Edit control
 	// TODO: fix Spell dlg so we don't rely on this
@@ -679,15 +726,39 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 	_hideSplash();
 #endif
 	
-	while (GetMessage(&msg, NULL, 0, 0))
-	{
-		// Note: we do not call TranslateMessage() because
-		// Note: the keybinding mechanism is responsible
-		// Note: for deciding if/when to do this.
-			
-		DispatchMessage(&msg);
-	}
+	// Win32 does not put the program name in argv[0], so [0] is the first argument
+	int nFirstArg = 0;
+	int k;
 	
+	// quick & dirty check to make sure we didn't do a command-line conversion
+   for (k=nFirstArg; (k<Args.m_argc); k++) {
+		if (*Args.m_argv[k] == '-') {
+			if (UT_stricmp(Args.m_argv[k],"-to") == 0) {
+ 				bShowApp = UT_FALSE;
+			}
+		}
+	}	
+
+    for (k=nFirstArg; (k<Args.m_argc); k++) {
+		if (*Args.m_argv[k] == '-') {
+			if (UT_stricmp(Args.m_argv[k],"-show") == 0) {
+ 				bShowApp = UT_TRUE;
+			}
+		}
+	}	
+
+	if (bShowApp)
+	{
+		while (GetMessage(&msg, NULL, 0, 0))
+		{
+			// Note: we do not call TranslateMessage() because
+			// Note: the keybinding mechanism is responsible
+			// Note: for deciding if/when to do this.
+				
+			DispatchMessage(&msg);
+		}
+	}
+
 	// destroy the App.  It should take care of deleting all frames.
 
 	pMyWin32App->shutdown();
@@ -713,6 +784,9 @@ void AP_Win32App::ParseCommandLine(int iCmdShow)
 	int nFirstArg = 0;
 	int k;
 	int kWindowsOpened = 0;
+	char *to = NULL;
+	int verbose = 1;
+	UT_Bool show = UT_FALSE;
 	
 	for (k=nFirstArg; (k<m_pArgs->m_argc); k++)
 	{
@@ -736,10 +810,28 @@ void AP_Win32App::ParseCommandLine(int iCmdShow)
 				// dump the string table in english as a template for translators.
 				// see abi/docs/AbiSource_Localization.abw for details.
 				AP_BuiltinStringSet * pBuiltinStringSet = new AP_BuiltinStringSet(this,AP_PREF_DEFAULT_StringSet);
-				pBuiltinStringSet->dumpBuiltinSet("EnUS.strings");
+				pBuiltinStringSet->dumpBuiltinSet("en-US.strings");
 				delete pBuiltinStringSet;
 #endif
 			}
+			else if (UT_stricmp(m_pArgs->m_argv[k],"-nosplash") == 0)
+			{
+				// we've alrady processed this before we initialized the App class
+			}
+			else if (UT_stricmp (m_pArgs->m_argv[k],"-to") == 0)
+			{
+				k++;
+				to = m_pArgs->m_argv[k];
+			}
+			else if (UT_stricmp (m_pArgs->m_argv[k], "-show") == 0)
+			{
+				show = UT_TRUE;
+			}
+			else if (UT_stricmp (m_pArgs->m_argv[k], "-verbose") == 0)
+			{
+				k++;
+				verbose = atoi (m_pArgs->m_argv[k]);
+			}
 			else
 			{
 				UT_DEBUGMSG(("Unknown command line option [%s]\n",m_pArgs->m_argv[k]));
@@ -749,37 +841,52 @@ void AP_Win32App::ParseCommandLine(int iCmdShow)
 		else
 		{
 			// [filename]
-			
-			AP_Win32Frame * pFirstWin32Frame = new AP_Win32Frame(this);
-			pFirstWin32Frame->initialize();
-			if (pFirstWin32Frame->loadDocument(m_pArgs->m_argv[k], IEFT_Unknown))
+			if (to) 
 			{
-				kWindowsOpened++;
-
-				HWND hwnd = pFirstWin32Frame->getTopLevelWindow();
-				ShowWindow(hwnd, iCmdShow);
-				UpdateWindow(hwnd);
+				AP_Convert * conv = new AP_Convert();
+				conv->setVerbose(verbose);
+				conv->convertTo(m_pArgs->m_argv[k], to);
+				delete conv;
 			}
 			else
 			{
-				// TODO: warn user that we couldn't open that file
+				AP_Win32Frame * pFirstWin32Frame = new AP_Win32Frame(this);
+				pFirstWin32Frame->initialize();
+
+				UT_Error error = pFirstWin32Frame->loadDocument(m_pArgs->m_argv[k], IEFT_Unknown);
+				if (!error)
+				{
+					kWindowsOpened++;
+
+					HWND hwnd = pFirstWin32Frame->getTopLevelWindow();
+					ShowWindow(hwnd, iCmdShow);
+					UpdateWindow(hwnd);
+				}
+				else
+				{
+					// TODO: warn user that we couldn't open that file
 
 #if 1
-				// TODO we crash if we just delete this without putting something
-				// TODO in it, so let's go ahead and open an untitled document
-				// TODO for now.  this would cause us to get 2 untitled documents
-				// TODO if the user gave us 2 bogus pathnames....
-				kWindowsOpened++;
-				pFirstWin32Frame->loadDocument(NULL, IEFT_Unknown);
-				HWND hwnd = pFirstWin32Frame->getTopLevelWindow();
-				ShowWindow(hwnd, iCmdShow);
-				UpdateWindow(hwnd);
+					// TODO we crash if we just delete this without putting something
+					// TODO in it, so let's go ahead and open an untitled document
+					// TODO for now.  this would cause us to get 2 untitled documents
+					// TODO if the user gave us 2 bogus pathnames....
+					kWindowsOpened++;
+					pFirstWin32Frame->loadDocument(NULL, IEFT_Unknown);
+					HWND hwnd = pFirstWin32Frame->getTopLevelWindow();
+					ShowWindow(hwnd, iCmdShow);
+					UpdateWindow(hwnd);
 #else
-				delete pFirstWin32Frame;
+					delete pFirstWin32Frame;
 #endif
+				}
 			}
 		}
 	}
+					
+	// command-line conversion may not open any windows at all
+	if (to && !show)
+		return;
 
 	if (kWindowsOpened == 0)
 	{
