@@ -42,13 +42,13 @@
 
 void s_RTF_ListenerWriteDoc::_closeSection(void)
 {
-	m_apiThisSection = NULL;
+	m_apiThisSection = 0;
 	return;
 }
 
 void s_RTF_ListenerWriteDoc::_closeBlock(void)
 {
-	m_apiThisBlock = NULL;
+	m_apiThisBlock = 0;
 	return;
 }
 
@@ -87,7 +87,8 @@ void s_RTF_ListenerWriteDoc::_openSpan(PT_AttrPropIndex apiSpan)
 	if (ndxColor != 0)
 		m_pie->_rtf_keyword("cf",ndxColor);
 
-	UT_sint32 ndxFont = m_pie->_findFont(&_rtf_font_info(pSpanAP,pBlockAP,pSectionAP));
+   	_rtf_font_info fi(pSpanAP,pBlockAP,pSectionAP);
+	UT_sint32 ndxFont = m_pie->_findFont(&fi);
 	UT_ASSERT(ndxFont != -1);
 	m_pie->_rtf_keyword("f",ndxFont);	// font index in fonttbl
 
@@ -110,6 +111,8 @@ void s_RTF_ListenerWriteDoc::_openSpan(PT_AttrPropIndex apiSpan)
 	{
 		if (strstr(szFontDecoration,"underline") != 0)
 			m_pie->_rtf_keyword("ul");
+		if (strstr(szFontDecoration,"overline") != 0)
+			m_pie->_rtf_keyword("ol");
 		if (strstr(szFontDecoration,"line-through") != 0)
 			m_pie->_rtf_keyword("strike");
 	}
@@ -244,8 +247,8 @@ s_RTF_ListenerWriteDoc::s_RTF_ListenerWriteDoc(PD_Document * pDocument,
 	m_pie = pie;
 	m_bInSpan = UT_FALSE;
 	m_apiLastSpan = 0;
-	m_apiThisSection = NULL;
-	m_apiThisBlock = NULL;
+	m_apiThisSection = 0;
+	m_apiThisBlock = 0;
 
 	m_bToClipboard = bToClipboard;
 	// when we are going to the clipboard, we should implicitly
@@ -554,7 +557,7 @@ void s_RTF_ListenerWriteDoc::_rtf_open_block(PT_AttrPropIndex api)
 	m_pie->_rtf_keyword_ifnotdefault_twips("sb",szTopMargin,0);
 	m_pie->_rtf_keyword_ifnotdefault_twips("sa",szBottomMargin,0);
 
-	if (UT_strcmp(szLineHeight,"1.0") != 0)
+	if (strcmp(szLineHeight,"1.0") != 0)
 	{
 		double f = UT_convertDimensionless(szLineHeight);
 		if (f != 0.0)					// we get zero on bogus strings....
