@@ -67,6 +67,7 @@ RTFProps_CharProps::RTFProps_CharProps()
 	m_bold = UT_FALSE;
 	m_italic = UT_FALSE;
 	m_underline = UT_FALSE;
+	m_overline = UT_FALSE;
 	m_strikeout = UT_FALSE;
 	m_superscript = UT_FALSE;
 	m_subscript = UT_FALSE;
@@ -201,7 +202,7 @@ IE_Imp_RTF::~IE_Imp_RTF()
 }
 
 
-IEStatus IE_Imp_RTF::importFile(const char * szFilename)
+UT_Error IE_Imp_RTF::importFile(const char * szFilename)
 {
 	m_newParaFlagged = UT_TRUE;
 	m_newSectionFlagged = UT_TRUE;
@@ -210,28 +211,28 @@ IEStatus IE_Imp_RTF::importFile(const char * szFilename)
 	if (!fp)
 	{
 		UT_DEBUGMSG(("Could not open file %s\n",szFilename));
-		return IES_FileNotFound;
+		return UT_IE_FILENOTFOUND;
 	}
 	
-	IEStatus iestatus = _writeHeader(fp);
-	if (iestatus == IES_OK)
+	UT_Error error = _writeHeader(fp);
+	if (!error)
 	{
-		iestatus = _parseFile(fp);
+		error = _parseFile(fp);
 	}
 
 	fclose(fp);
 	
-	return iestatus;
+	return error;
 }
 
 
-IEStatus IE_Imp_RTF::_writeHeader(FILE * /*fp*/)
+UT_Error IE_Imp_RTF::_writeHeader(FILE * /*fp*/)
 {
-		return IES_OK;
+		return UT_OK;
 }
 
 
-IEStatus IE_Imp_RTF::_parseFile(FILE* fp)
+UT_Error IE_Imp_RTF::_parseFile(FILE* fp)
 {
 	m_pImportFile = fp;
 
@@ -310,25 +311,38 @@ IEStatus IE_Imp_RTF::_parseFile(FILE* fp)
 
 	if (ok)
 	{
-		ok = FlushStoredChars();
+		ok = FlushStoredChars(UT_TRUE);
 	}
-	return ok ? IES_OK : IES_Error;
+	return ok ? UT_OK : UT_ERROR;
 }
 
 /*****************************************************************/
 /*****************************************************************/
 
+UT_Bool IE_Imp_RTF::RecognizeContents(const char * szBuf, int iNumbytes)
+{
+	if ( iNumbytes < 5 )
+	{
+		return(UT_FALSE);
+	}
+	if ( strncmp( szBuf, "{\\rtf", 5 ) == 0 )
+	{
+		return(UT_TRUE) ;
+	}
+	return(UT_FALSE);
+}
+
 UT_Bool IE_Imp_RTF::RecognizeSuffix(const char * szSuffix)
 {
 	return (UT_stricmp(szSuffix,".rtf") == 0);
 }
 
-IEStatus IE_Imp_RTF::StaticConstructor(PD_Document * pDocument,
+UT_Error IE_Imp_RTF::StaticConstructor(PD_Document * pDocument,
 										IE_Imp ** ppie)
 {
 	IE_Imp_RTF * p = new IE_Imp_RTF(pDocument);
 	*ppie = p;
-	return IES_OK;
+	return UT_OK;
 }
 
 UT_Bool	IE_Imp_RTF::GetDlgLabels(const char ** pszDesc,
@@ -506,7 +520,7 @@ UT_Bool IE_Imp_RTF::ParseChar(UT_UCSChar ch)
 				return UT_TRUE;
 			}
 			// Insert a character into the story
-			if ((ch >= 32  ||  ch == 9)  &&  !m_currentRTFState.m_charProps.m_deleted)
+            if ((ch >= 32  ||  ch == 9 || ch == UCS_FF || ch == UCS_LF)  &&  !m_currentRTFState.m_charProps.m_deleted)
 			{
 				return AddChar(ch);
 			}
@@ -543,7 +557,7 @@ UT_Bool IE_Imp_RTF::ReadKeyword(unsigned char* pKeyword, long* pParam, UT_Bool*
 
 	// Read the first character of the control word
 	unsigned char ch;
-	if (!ReadCharFromFile(&ch))
+	if (!ReadCharFromFileWithCRLF(&ch))
 		return UT_FALSE;
 
 	// If it's a control symbol there is no delimiter, its just one character
@@ -559,7 +573,7 @@ UT_Bool IE_Imp_RTF::ReadKeyword(unsigned char* pKeyword, long* pParam, UT_Bool*
 	{
 		*pKeyword = ch;
 		pKeyword++;
-		if (!ReadCharFromFile(&ch))
+		if (!ReadCharFromFileWithCRLF(&ch))
 			return UT_FALSE;
 	}
 	*pKeyword = 0;
@@ -568,7 +582,7 @@ UT_Bool IE_Imp_RTF::ReadKeyword(unsigned char* pKeyword, long* pParam, UT_Bool*
     if (ch == '-')
     {
         fNegative = UT_TRUE;
-		if (!ReadCharFromFile(&ch))
+		if (!ReadCharFromFileWithCRLF(&ch))
 			return UT_FALSE;
     }
 
@@ -579,7 +593,7 @@ UT_Bool IE_Imp_RTF::ReadKeyword(unsigned char* pKeyword, long* pParam, UT_Bool*
 		while (isdigit(ch))
 		{
 			parameter[count++] = ch;
-			if (!ReadCharFromFile(&ch))
+			if (!ReadCharFromFileWithCRLF(&ch))
 				return UT_FALSE;
 		}
 		parameter[count] = 0;
@@ -589,8 +603,8 @@ UT_Bool IE_Imp_RTF::ReadKeyword(unsigned char* pKeyword, long* pParam, UT_Bool*
 			*pParam = -*pParam;
 	}
 
-	// If the delimeter was non-space then this character is part of the following text!
-	if (ch != ' ')
+	// If the delimeter was non-whitespace then this character is part of the following text!
+	if ((ch != ' ') && (ch != 10) && (ch != 13))
 	{
 		SkipBackChar(ch);
 	}
@@ -599,36 +613,44 @@ UT_Bool IE_Imp_RTF::ReadKeyword(unsigned char* pKeyword, long* pParam, UT_Bool*
 }
 
 
-UT_Bool IE_Imp_RTF::ReadCharFromFile(unsigned char* pCh)
+// Reads a character from the file. Doesn't ignore CR and LF
+UT_Bool IE_Imp_RTF::ReadCharFromFileWithCRLF(unsigned char* pCh)
 {
+	
+	UT_Bool ok = UT_FALSE;
+
 	if (m_pImportFile)					// if we are reading a file
 	{
-		while (fread(pCh, 1, sizeof(unsigned char), m_pImportFile) > 0)
+		if (fread(pCh, 1, sizeof(unsigned char), m_pImportFile) > 0)
 		{
-			// line feed and cr should be ignored in RTF files
-			if (*pCh != 10  &&  *pCh != 13)
-			{
-				return UT_TRUE;
-			}
-		}
-
-		return UT_FALSE;
+			ok = UT_TRUE;
+		}		
 	}
 	else								// else we are pasting from a buffer
 	{
-		while (m_pCurrentCharInPasteBuffer < m_pPasteBuffer+m_lenPasteBuffer)
+		if (m_pCurrentCharInPasteBuffer < m_pPasteBuffer+m_lenPasteBuffer)
 		{
 			*pCh = *m_pCurrentCharInPasteBuffer++;
-			
-			// line feed and cr should be ignored in RTF files
-			if (*pCh != 10  &&  *pCh != 13)
-			{
-				return UT_TRUE;
-			}
+			ok = UT_TRUE;
 		}
-
-		return UT_FALSE;
 	}
+
+	return ok;
+}
+
+// Reads a character from the file ignoring CR and LF
+UT_Bool IE_Imp_RTF::ReadCharFromFile(unsigned char* pCh)
+{
+	// line feed and cr should be ignored in RTF files
+	do
+	{
+		if (ReadCharFromFileWithCRLF(pCh) == UT_FALSE) {
+			return UT_FALSE;
+		}
+	} while (*pCh == 10  ||  *pCh == 13);
+	
+	return UT_TRUE;
+
 }
 
 
@@ -756,8 +778,17 @@ UT_Bool IE_Imp_RTF::TranslateKeyword(unsigned char* pKeyword, long param, UT_Boo
 		{
 			m_currentRTFState.m_paraProps.m_indentLeft = param;
 		}
+		else if (strcmp((char*)pKeyword, "line") == 0)
+		{
+			return ParseChar(UCS_LF);
+		}	
+		break;
+	case 'o': 
+	        if (strcmp((char*)pKeyword,"ol") == 0)
+	        {
+		         return HandleOverline(fParam ? param : 1);
+		}
 		break;
-
 	case 'p':
 		if (strcmp((char*)pKeyword, "par") == 0)
 		{
@@ -774,7 +805,11 @@ UT_Bool IE_Imp_RTF::TranslateKeyword(unsigned char* pKeyword, long param, UT_Boo
 			// reset paragraph attributes
 			return ResetParagraphAttributes();
 		}
-		break;
+	        else if (strcmp((char*)pKeyword, "page") == 0)
+       		{
+           	 return ParseChar(UCS_FF);
+        	}
+			break;
 
 	case 'q':
 		if (strcmp((char*)pKeyword, "ql") == 0)
@@ -914,7 +949,32 @@ UT_Bool IE_Imp_RTF::TranslateKeyword(unsigned char* pKeyword, long param, UT_Boo
 	case '*':
 		if (strcmp((char*)pKeyword, "*") == 0)
 		{
-			// TODO different destination (all unhandled at the moment, so enter skip mode)
+		  //
+		  // Code to handle overline importing.
+		  //
+		  unsigned char keyword_star[256];
+		  long parameter_star = 0;
+		  UT_Bool parameterUsed_star = UT_FALSE;
+		  //
+		  // Look for \*\ol sequence. Ignore all others
+		  // 
+		  if (ReadKeyword(keyword_star, &parameter_star, &parameterUsed_star))
+		    {
+		      if( strcmp((char*)keyword_star, "\\")== 0)
+                        {
+		          if (ReadKeyword(keyword_star, &parameter_star, &parameterUsed_star))
+		            { 		
+      		               if( strcmp((char*)keyword_star,"ol") == 0)
+			         { 
+			            return HandleOverline(parameterUsed_star ? 
+					      parameter_star : 1);
+                                 }
+                            }
+			}
+		    }
+
+// Ignore all other \* tags
+// TODO different destination (all unhandled at the moment, so enter skip mode)
 			m_currentRTFState.m_destinationState = RTFStateStore::rdsSkip;
 		}
 		break;
@@ -956,20 +1016,36 @@ UT_Bool IE_Imp_RTF::ApplyCharacterAttributes()
 	// italic
 	strcat(propBuffer, "; font-style:");
 	strcat(propBuffer, m_currentRTFState.m_charProps.m_italic ? "italic" : "normal");
-	// underline & strike-out
+	// underline & overline & strike-out
 	strcat(propBuffer, "; text-decoration:");
-	if (m_currentRTFState.m_charProps.m_underline  &&  m_currentRTFState.m_charProps.m_strikeout)
+	if (m_currentRTFState.m_charProps.m_underline  &&  m_currentRTFState.m_charProps.m_strikeout && !m_currentRTFState.m_charProps.m_overline)
 	{
 		strcat(propBuffer, "underline line-through");
 	}
-	else if (m_currentRTFState.m_charProps.m_underline  &&  !m_currentRTFState.m_charProps.m_strikeout)
+	else if (m_currentRTFState.m_charProps.m_underline  &&  !m_currentRTFState.m_charProps.m_strikeout  && !m_currentRTFState.m_charProps.m_overline)
 	{
 		strcat(propBuffer, "underline");
 	}
-	else if (!m_currentRTFState.m_charProps.m_underline  &&  m_currentRTFState.m_charProps.m_strikeout)
+	else if (!m_currentRTFState.m_charProps.m_underline  &&  m_currentRTFState.m_charProps.m_strikeout  && !m_currentRTFState.m_charProps.m_overline)
 	{
 		strcat(propBuffer, "line-through");
 	}
+	else if(m_currentRTFState.m_charProps.m_underline  &&  m_currentRTFState.m_charProps.m_strikeout && m_currentRTFState.m_charProps.m_overline)
+	{
+		strcat(propBuffer, "underline overline line-through");
+	}
+	else if(!m_currentRTFState.m_charProps.m_underline  &&  m_currentRTFState.m_charProps.m_strikeout && m_currentRTFState.m_charProps.m_overline)
+	{
+		strcat(propBuffer, "overline line-through");
+	}
+	else if(m_currentRTFState.m_charProps.m_underline  &&  !m_currentRTFState.m_charProps.m_strikeout && m_currentRTFState.m_charProps.m_overline)
+	{
+		strcat(propBuffer, "underline overline");
+	}
+	else if(!m_currentRTFState.m_charProps.m_underline  &&  !m_currentRTFState.m_charProps.m_strikeout && m_currentRTFState.m_charProps.m_overline)
+	{
+		strcat(propBuffer, "overline");
+	}
 	else
 	{
 		strcat(propBuffer, "none");
@@ -1533,6 +1609,11 @@ UT_Bool IE_Imp_RTF::HandleUnderline(UT_Bool state)
 	return HandleBoolCharacterProp(state, &m_currentRTFState.m_charProps.m_underline);
 }
 
+UT_Bool IE_Imp_RTF::HandleOverline(UT_Bool state)
+{
+	return HandleBoolCharacterProp(state, &m_currentRTFState.m_charProps.m_overline);
+}
+
 UT_Bool IE_Imp_RTF::HandleStrikeout(UT_Bool state)
 {
 	return HandleBoolCharacterProp(state, &m_currentRTFState.m_charProps.m_strikeout);
@@ -1629,3 +1710,8 @@ void IE_Imp_RTF::pasteFromBuffer(PD_DocumentRange * pDocRange,
 	return;
 }
 
+
+
+
+
+
